@@ -115,6 +115,8 @@ def getcti_ts06(nshot):
         cdev = cti.getdevice('TS6')
         ts06 = cdev['TRIGGER']
         logger.debug('getcti_ts06 %d', ts06)
+    if ts06 < 1e15:
+        ts06 = None
     return ts06
 
 
@@ -473,7 +475,6 @@ class SFREAD:
 
         pset = str_byt.to_str(pset)
         sfo = self.sfh[pset]
-        logger.debug('ParSet %s %d', pset, sfo.length)
         otyp = sfo.obj_type
         if otyp not in (3, 4):
             return None
@@ -482,23 +483,24 @@ class SFREAD:
 
         j0 = 0
         par_d = {}
-        
+        logger.debug('Diag: %s, PS: %s, addr: %d, length: %d', self.diag, pset, sfo.items, sfo.length)
         for j in range(sfo.items):
             pname = str_byt.to_str(buf[j0: j0+8])
-            unit, dfmt, n_items = unpack('>3H', buf[j0+8:  j0+14])
+            unit, dfmt, n_items = unpack('>3h', buf[j0+8:  j0+14])
             meta = type('', (), {})()
 #            meta.phys_unit = sfdics.unit_d[unit]
             meta.data_format = dfmt
-            logger.debug('%s %d %d %d', pname, unit, dfmt, n_items)
             status = unpack('>h', buf[j0+14:  j0+16])[0]
+            logger.debug('PN: %s, j0: %d, unit: %d dfmt: %d n_items: %d, status: %d', pname, j0, unit, dfmt, n_items, status)
             j0 += 16
+
             if dfmt in sfdics.fmt2len.keys(): # char variable
                 dlen = sfdics.fmt2len[dfmt]
                 bytlen = n_items * dlen
                 data = np.chararray((n_items,), itemsize=dlen, buffer=buf[j0+2: j0+2+bytlen])
                 dj0 = 8 * ( (bytlen + 9)//8 )
                 j0 += dj0
-            else: # number
+            elif dfmt in sfdics.fmt2typ.keys(): # number
                 sfmt = sfdics.fmt2struc[dfmt]
                 val_len = n_items + 2
                 bytlen = val_len * np.dtype(sfmt).itemsize
@@ -510,11 +512,12 @@ class SFREAD:
                     data = np.squeeze(data[2:])
                 dj0 = 8 * ( (bytlen + 7)//8 )
                 j0 += dj0
-
-            logger.debug('PS %s, PN %s', pset, pname)
-            print(data)
+            else: # faulty dfmt
+                break
             par_d[pname] = sfobj.SFOBJ(data, sfho=meta)
 
+            if j0 >= sfo.length:
+                break
         return par_d
 
 
@@ -578,22 +581,53 @@ class SFREAD:
         return self.getparset(lbl)
 
 
+    def get_ts06(self):
+
+        t606 = None
+        for obj, val in self.sfh.items():
+            if hasattr(val, 'obj_type'):
+                if val.obj_type == 3:
+                    ps = self.getparset(obj)
+                    if 'TS06' in ps.keys():
+                        ts6 = ps['TS06']
+                        if ts6 > 1e15:
+                            ts06 = ts6
+                            break
+        return ts06
+
+
     def getts06(self, obj):
         """
         Reads the diagnostic internal TS06 from parameter set
         """
-        ts6 = 0
-        for obj in self.getrelnames(obj):
-            if self.sfh[obj].obj_type == 3:
-                ps = self.getparset(obj)
+        ts06 = None
+        if self.sfh[obj].obj_type == 8:
+            tb = obj
+        else:
+            for rel_obj in self.getrelnames(obj):
+                if self.sfh[rel_obj].obj_type == 8: # related TB
+                    tb = rel_obj
+                    break
+        if 'tb' in locals(): # No TB related
+            obj2 = tb
+        else: # try a direct relation
+            obj2 = obj
+
+        for rel_obj in self.getrelnames(obj2):
+            if self.sfh[rel_obj].obj_type == 3: # related device
+                ps = self.getparset(rel_obj)
                 if 'TS06' in ps.keys():
                     ts6 = ps['TS06']
-                    break
+                    if ts6 > 1e15:
+                        ts06 = ts6
+                        break
 
-        logger.debug('getts06 %s %d', obj, ts6)
-        if ts6 < 1e15:
-            ts6 = getcti_ts06(self.shot)
-        return ts6
+        logger.debug('getts06 %s %d', rel_obj, ts06)
+        if ts06 is None:
+            ts06 = self.get_ts06()
+        if ts06 is None:
+            ts06 = getcti_ts06(self.shot)
+        return ts06
 
 
     def time_first(self, obj):
